// Global app controller

//API Key: 497663938924fcd19c9fd2570cf76c7c
//http://food2fork.com/api/search


import Search from './models/Search';
import Recipe from './models/Recipe';
import List from './models/List';
import Likes from './models/Likes';
import * as searchView from './views/searchView'; //object in which all the imported variables from searchView will be stored 
import * as recipeView from './views/recipeView';
import * as listView from './views/listView';
import * as likesView from './views/likesView';
import { elements, renderLoader, clearLoader } from './views/base';

/** Global state of the app
    - Search Object
    - Current Recipe Object
    - Shoping list object
    - liked recipes 
*/
const state = {};
//window.state = state;

//SEARCH CONTROLLER
const controlSearch = async () => {
    //1.) Get query from view 
    const query = searchView.getInput();
    //const query = 'pizza';
    //console.log(query);
    
    if (query){
        //2.) New search object and add to state
        state.search = new Search(query);
        
        //3.) Prepare UI for results
        searchView.clearInput();
        searchView.clearResults();
        renderLoader(elements.searchRes);
        
        try{
            //4.) Search for recipes
            await state.search.getResults();


            //5.) Render results on UI
            clearLoader();
            searchView.renderResults(state.search.result);
            
            
        }catch(error){
            alert(`Something went wrong with the search.`);
            clearLoader();
        }
 

    }
};


elements.searchForm.addEventListener('submit', e => {
    e.preventDefault();
    controlSearch();
});



////TESTING
//window.addEventListener('load', e => {
//    e.preventDefault();
//    controlSearch();
//});


elements.searchResPages.addEventListener('click', e => {
    const btn = e.target.closest('.btn-inline');
    if (btn) {
        const goToPage = parseInt(btn.dataset.goto, 10);
        searchView.clearResults();
        searchView.renderResults(state.search.result, goToPage);
        //console.log(goToPage);
    }
});




//RECIPE CONTROLLER
//***************



const controlRecipe = async () => {
    //1.) Get ID from URL
    const id = window.location.hash.replace('#','');
    //console.log(id);
    
    //just making sure an ID has been selected
    if(id){
        
        //2.) Prepare UI for changes
        recipeView.clearRecipe();
        renderLoader(elements.recipe);
        
        
        //Highlight selected search item 
        if (state.search)searchView.highlightSelected(id);
        // if because we get an error if there is nothing to search so there must be a search
        
        
        //3.) Create a new recipe object 
        state.recipe = new Recipe(id);
        
        
        try{
            
            //4.) Get Recipe data and parse ingredients
            await state.recipe.getRecipe(); 
            //^(because async func, this loads in background)
            
            state.recipe.parseIngredients();

            //5.) Calculate servings and time
            state.recipe.calcTime();
            state.recipe.calcServings();
            

            //6.) Render the recipe 
            clearLoader();
            recipeView.renderRecipe(
                state.recipe,
                state.likes.isLiked(id)
                //^ pass this in to tell us if it is liked or not
            );
            
            
        }catch(error){
            console.log(error);
            alert(`Something went wrong, Error processing recipe.`);
    }
        //console.log(state.recipe);

    }
};


//window.addEventListener('hashchange', controlRecipe);
//window.addEventListener('load', controlRecipe);

['hashchange', 'load'].forEach(event => window.addEventListener(event, controlRecipe));




//LIST CONTROLLER
//***************

const controlList = () => {
    
    console.log('Add To Cart - Pressed');
    //Add to cart button pressed
    
    // Create a new list IF there is none yet
    if (!state.list) state.list = new List();
    //^we don't pass anything into the list because it is empty, we just want to initialize it with an empty object
    
    
    // Add each ingredient to the list and UI 
    state.recipe.ingredients.forEach(el => {
        //^state.recipe.ingredients is an array and each item in that array will next be added to the list
        
        const item = state.list.addItem(el.count, el.unit, el.ingredient);
        //^using the addItem method from the List.js model and we pass all the data into the method nad it will RETURN an the item object to us and we set that equal to the item constant at the beginning of this statement.  
        
        
        listView.renderItem(item);
        //^using the renderItem function from listView.js, we pass in the item and it reads all of the data that it needs and prints it to the UI 
    })
}



//Handle, delete and update, list item events 
//(Changing the count on shopping list and deleting items off the list)
elements.shopping.addEventListener('click', e => {
    
    const id = e.target.closest('.shopping__item').dataset.itemid;
    //^if the click occurs anywhere in that area .closest, in the above declaration, will record the id from the closest shopping item to the click event (also, we use '.itemid' because in listView.js that's what its named in the markup '<li class="shopping__item" data-itemid=${item.id}>')
    
    
    //Handle the delete event / delete button (little red x)
    if(e.target.matches('.shopping__delete, .shopping__delete *')) {
        //^ we use .matches because it will return a T/F value 
        
        //console.log('.shopping__delete - pressed');
        
        //and if the above is true the item is deleted using the following:
        
        // Delete from state
        state.list.deleteItem(id);
        
        //Delete from UI
        listView.deleteItem(id);
        
        
    } else if (e.target.matches('.shopping__count-value')) {
        //looking for the event where the user clicks on the little up/down button for the count of the ingredient in the list 
        
        //console.log('shopping__count-value - pressed');
        
        const val = parseFloat(e.target.value, 10);
        
        state.list.updateCount(id, val);
    }
});




//LIKES CONTROLLER
//***************

//console.log(state.likes);
//^ For testing (remove for production)



const controlLike = () => {
    if(!state.likes) state.likes = new Likes();
    
    const currentID = state.recipe.id;
    //^Gets us the ID we need
    //console.log(state.likes);
    
    
    if(!state.likes.isLiked(currentID)){
        //^User has not liked the current recipe
        
        const newLike = state.likes.addLike(
            currentID,
            state.recipe.title,
            state.recipe.author,
            state.recipe.img
        );
        //^Add Like to state
        
        likesView.toggleLikeBtn(true);
        //^ Toggle the Like Button
        
        
        likesView.renderLike(newLike);        
        //^ Add Liked item to the UI list
        
    }else{
        //^User has liked the current recipe
        
        state.likes.deleteLike(currentID);
        //^Remove like from the state
        
        likesView.toggleLikeBtn(false);
        //^Toggle the like button
        
        
        likesView.deleteLike(currentID);
        //console.log(currentID, 'index - controlLike');
        //^Remove like from the UI
    }
    
    likesView.toggleLikeMenu(state.likes.getNumLikes());
}



// Restore Liked Recipes on page load - using local storage
window.addEventListener('load', () => {
    
    state.likes = new Likes();
    //^create a new empty likes object
    
    state.likes.readStorage();
    //^ Restore Likes - reads local storage and fills the likes object with prevoiusly liked recipe objects
    
    likesView.toggleLikeMenu(state.likes.getNumLikes());
    //^Toggles like menu button
    
    state.likes.likes.forEach(like => likesView.renderLike(like));
    //^Render the existing likes
});








//Handling Recipe Button clicks - all events that occur in the recipe area are handled here
elements.recipe.addEventListener('click', e => {
   // we cannot use e.target.closes becuse we have different elements that we want to click on 
    if (e.target.matches('.btn-decrease, .btn-decrease *')) {
        //Decrease button is clicked
        if (state.recipe.servings > 1) {
            
            state.recipe.updateServings('dec');
            
            recipeView.updateServingsIngredients(state.recipe);
            
        }
        
    } else if (e.target.matches('.btn-increase, .btn-increase *')) {
        //Increase button is clicked
        
        state.recipe.updateServings('inc');
        
        recipeView.updateServingsIngredients(state.recipe);
        
    } else if (e.target.matches('.recipe__btn--add, .recipe__btn--add *')) {
        // Add ingredients to shopping list
        controlList();
        
    } else if (e.target.matches(".recipe__love, .recipe__love *")) {
        // Like controller
        controlLike();
    }
    
    
});


//window.l = new List();
//^Dev purposed don't need 


























































////first way to import (only one item);
//// when importing a model we don't have to specify .js
//import str from './models/Search';
//
////2nd way to import items 
////import multiple things from a module (use 'as' to change variable name)
////import {add as a, multiply as m, ID} from './views/searchView';
//
//
////3rd way to import items (importing all of them)
//import * as searchView from './views/searchView';
//
//console.log(`Using imported functions! ${searchView.add(ID, 2)} and ${searchView.multiply(3,5)}.  ${str}`);