// npm items don't need a file path, just the name 
import axios from 'axios';
import { key, proxy } from '../config';
 
export default class Recipe {
    constructor(id) {
        this.id = id;
    }
    
    
    async getRecipe() {
        try{
            const res = await axios(`${proxy}http://food2fork.com/api/get?key=${key}&rId=${this.id}`);
            
            this.title = res.data.recipe.title;
            this.author = res.data.recipe.publisher;
            this.img = res.data.recipe.image_url;
            this.url = res.data.recipe.source_url;
            this.ingredients = res.data.recipe.ingredients;
            
            //console.log(res);
            
        } catch (error) {
            console.log(error);
            alert(`Something went wrong :( 
            Error Message: ${error}
            `);
        }
    }
    
    calcTime() {
        //assuming that we need 15 min for eash 3 ingredients 
        const numIng = this.ingredients.length;
        const periods = Math.ceil(numIng / 3 );
        this.time = periods * 15;
    }
    
    calcServings() {
        // Just saying that we have 4 servings on each of the recipes 
        this.servings = 4;
    }
    
    parseIngredients(){
        const unitsLong = ['tablespoons', 'tablespoon', 'ounce', 'ounces', 'teaspoon', 'teapoons', 'cups', 'pounds'];
        const unitsShort = ['tbsp', 'tbsp', 'oz', 'ozs', 'tsp',  'cup', 'pound'];
        const units = [...unitsShort, 'kg', 'g'];
        
        const newIngredient = this.ingredients.map(el => {
            // 1) uniform units
            let ingredient = el.toLowerCase();
            unitsLong.forEach((unit, i) => {
                ingredient = ingredient.replace(unit, unitsShort[i]);
            });
            
            
            // 2) Remove parentheses
            ingredient = ingredient.replace(/ *\([^)]*\) */g, ' ');
            
            
            // 3) Parse ingredients into count, unit and ingredient 
            const arrIng = ingredient.split(' '); 
            const unitIndex = arrIng.findIndex(el2 => units.includes(el2));
            
            let objIng;
            if (unitIndex > -1) {
                // There is a unit 
                //Ex: 4 1/2 cups, arrCount will be [4, 1/2] ---> eval('4+1/2') --> 4.5
                //Ex: 4 cups, arrCount will be [4]
                const arrCount = arrIng.slice(0, unitIndex);
                
                let count;
                if(arrCount.length === 1) {
                    count = arrIng[0].replace('-', '+');
                } else {
                    count = eval(arrIng.slice(0, unitIndex).join('+'));
                }
                
                objIng = {
                    count,
                    unit: arrIng[unitIndex],
                    ingredient: arrIng.slice(unitIndex + 1).join(' ')
                };
                
                
                
            } else if (parseInt(arrIng[0], 10)){
                //There is NO unit, but the first element is a number 
                 objIng = {
                    count: parseInt(arrIng[0], 10),
                    unit: '',
                    ingredient: arrIng.slice(1).join(' ')
                }
            } else if (unitIndex === -1) {
                // There is NO unit
                objIng = {
                    count: 5,
                    unit: '',
                    ingredient
                }
            }
            
            return objIng;
            
        });
        this.ingredients = newIngredient;
        
    }
    
    updateServings (type) {
        //parameter type helps us determine if we are increasing or decreasing
        
        //Servings
        const newServings = type === 'dec' ? this.servings - 1 : this.servings + 1;
        // ^ the value is stored in newServings
        
        //Ingredients - update all the count numbers for the ingredients
        this.ingredients.forEach(ing => {
            //ing is current value in forEach loop
            //ing.count = ing.count * (newServings / this.servings);
            ing.count *= (newServings / this.servings);
        });
        
        
        this.servings = newServings;
    }
}










