//export const add = (a, b) => a + b;
//export const multiply = (a, b) => a * b;
//export const ID = 23;

import {elements} from './base';

export const getInput = () => elements.searchInput.value;

export const clearInput = () => {
    //not returning anything in this func so it's cleaner to wrap it as an object
    elements.searchInput.value = '';
};

//clears the results list each time a new item is searched 
export const clearResults = () => {
    elements.searchResList.innerHTML = '';
    
    //clears the buttons
    elements.searchResPages.innerHTML = '';
};


// Highlights and unhighlights the selected recipe in the column on the left of the screen 
export const highlightSelected = id => {
    
    const resultsArr = Array.from(document.querySelectorAll('.results__link'));
    // just creates an array from the results__link
    
    resultsArr.forEach(el => {
        el.classList.remove('results__link--active');
        // takes the class off all of the links in the array so it doesn't stick to old ones we don't want highlighted anymore
    });
    
    document.querySelector(`.results__link[href*="${id}"]`).classList.add('results__link--active');
    //Above^: the weird symbol, kinda like a css selector gets us the link with the # hash symbol and the id after -- then it adds the classnames/link to the classlist
        //also when we add a class to the class list we do not want the 'dot selector' (period in front of class name) ex: '.classList.add('results__link--active');'
};



//shortening the recipe title that return in the results list 
export const limitRecipeTitle = (title, limit = 17) => {
    const newTitle = [];
    if(title.length > limit){
        title.split(' ').reduce((acc, cur) => {
            if(acc + cur.length <= limit){
                newTitle.push(cur);
            }
            return acc + cur.length;
        }, 0);
        
        //return the result
        return `${newTitle.join(' ')}...`;
    }
    
    return title;
};


const renderRecipe = recipe => {
    const markup = `
        <li>
            <a class="results__link" href="#${recipe.recipe_id}">
                <figure class="results__fig">
                    <img src="${recipe.image_url}" alt="${recipe.title}">
                </figure>
                <div class="results__data">
                    <h4 class="results__name">${limitRecipeTitle(recipe.title)}</h4>
                    <p class="results__author">${recipe.publisher}</p>
                </div>
            </a>
        </li>
    `;
    elements.searchResList.insertAdjacentHTML('beforeend', markup);
};


////tupe: 'prev' or 'next'
//const createButton = (page, type) => `
//         <button class="btn-inline results__btn--${type}" data-goto=${type === 'prev' ? page - 1 : page + 1}>
//            <svg class="search__icon">
//                <use href="img/icons.svg#icon-triangle-${type === 'prev' ? 'left' : 'right'}"></use>
//            </svg>
//            <span>Page ${type === 'prev' ? page - 1 : page + 1}</span>
//        </button>       
//`;

// type: 'prev' or 'next'
const createButton = (page, type) => `
    <button class="btn-inline results__btn--${type}" data-goto=${type === 'prev' ? page - 1 : page + 1}>
        <span>Page ${type === 'prev' ? page - 1 : page + 1}</span>
        <svg class="search__icon">
            <use href="img/icons.svg#icon-triangle-${type === 'prev' ? 'left' : 'right'}"></use>
        </svg>
    </button>
`;


//render buttons according to the page that we are on 
const renderButtons = (page, numResults, resPerPage) => {
    const pages = Math.ceil(numResults / resPerPage);
    
    let button; 
    
    if(page === 1 && pages > 1){
        //button to go to next page
        button = createButton(page, 'next');
    } else if(page < pages){
        // Both Buttons
        button = `
            ${createButton(page, 'prev')}
            ${createButton(page, 'next')}
        `;
            
    } else if(page === pages && pages > 1) {
        // only want button to go to previous page
        button = createButton(page, 'prev');
    }
    
    elements.searchResPages.insertAdjacentHTML('afterbegin', button);
};



export const renderResults = (recipes, page = 1, resPerPage = 10) => {
    // render results of current page
    const start = (page - 1) * resPerPage;
    const end = page * resPerPage;
    
    recipes.slice(start, end).forEach(renderRecipe);
    
    //render pagination buttons
    renderButtons(page, recipes.length, resPerPage);
};












